@@ -18,9 +18,16 @@ s.rstrip("e")  # no warning
 s.rstrip("\n\t ")  # no warning
 s.rstrip(r"\n\t ")  # warning
 
-from somewhere import strip, other_type
+from somewhere import other_type, strip
 
 strip("we")  # no warning
 other_type().lstrip()  # no warning
 other_type().rstrip(["a", "b", "c"])  # no warning
 other_type().strip("a", "b")  # no warning
+
+import test, test2  # isort: skip
+import test_as as test3
+
+test.strip("test")  # no warning
+test2.strip("test")  # no warning
+test3.strip("test")  # no warning
