@@ -8,9 +8,6 @@ import unittest
 from argparse import Namespace
 from pathlib import Path
 
-from hypothesis import HealthCheck, given, settings
-from hypothesmith import from_grammar
-
 from bugbear import (
     B001,
     B002,
@@ -39,6 +36,16 @@ from bugbear import (
     B025,
     B026,
     B027,
+    B028,
+    B029,
+    B030,
+    B031,
+    B032,
+    B033,
+    B034,
+    B035,
+    B036,
+    B037,
     B901,
     B902,
     B903,
@@ -46,6 +53,8 @@ from bugbear import (
     B905,
     B906,
     B907,
+    B908,
+    B909,
     B950,
     BugBearChecker,
     BugBearVisitor,
@@ -63,9 +72,8 @@ class BugbearTestCase(unittest.TestCase):
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
         expected = self.errors(
-            B001(8, 0, vars=("bare `except:`",)),
-            B001(40, 4, vars=("bare `except:`",)),
-            B001(47, 0, vars=("`except ():`",)),
+            B001(8, 0),
+            B001(40, 4),
         )
         self.assertEqual(errors, expected)
 
@@ -73,7 +81,7 @@ class BugbearTestCase(unittest.TestCase):
         filename = Path(__file__).absolute().parent / "b002.py"
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
-        self.assertEqual(errors, self.errors(B002(15, 8), B002(20, 11)))
+        self.assertEqual(errors, self.errors(B002(14, 8), B002(19, 11)))
 
     def test_b003(self):
         filename = Path(__file__).absolute().parent / "b003.py"
@@ -110,31 +118,31 @@ class BugbearTestCase(unittest.TestCase):
         self.assertEqual(
             errors,
             self.errors(
-                B006(60, 24),
-                B006(64, 29),
-                B006(68, 19),
-                B006(72, 19),
-                B006(76, 31),
-                B006(80, 25),
-                B006(85, 45 if sys.version_info >= (3, 8) else 46),
-                B008(85, 60),
-                B006(89, 45),
-                B008(89, 63),
-                B006(93, 44),
-                B008(93, 59),
-                B006(97, 32),
-                B008(109, 38),
-                B008(113, 11),
-                B008(113, 31),
-                B008(117, 29 if sys.version_info >= (3, 8) else 30),
-                B008(160, 29),
-                B008(164, 44),
-                B006(170, 19),
-                B008(170, 20),
-                B008(170, 30),
-                B008(176, 21),
-                B008(181, 18),
-                B008(181, 36),
+                B006(58, 24),
+                B006(61, 29),
+                B006(64, 19),
+                B006(67, 19),
+                B006(70, 31),
+                B006(73, 25),
+                B006(77, 45),
+                B008(77, 60),
+                B006(81, 45),
+                B008(81, 63),
+                B006(85, 44),
+                B008(85, 59),
+                B006(89, 32),
+                B008(100, 38),
+                B008(103, 11),
+                B008(103, 31),
+                B008(107, 29),
+                B008(149, 29),
+                B008(153, 44),
+                B006(159, 19),
+                B008(159, 20),
+                B008(159, 30),
+                B008(165, 21),
+                B008(170, 18),
+                B008(170, 36),
             ),
         )
 
@@ -164,7 +172,7 @@ class BugbearTestCase(unittest.TestCase):
 
         self.assertEqual(
             errors,
-            self.errors(B008(15, 66)),
+            self.errors(B008(13, 66)),
         )
 
     def test_b009_b010(self):
@@ -247,14 +255,22 @@ class BugbearTestCase(unittest.TestCase):
         filename = Path(__file__).absolute().parent / "b016.py"
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
-        expected = self.errors(B016(6, 0), B016(7, 0), B016(8, 0))
+        expected = self.errors(B016(6, 0), B016(7, 0), B016(8, 0), B016(10, 0))
         self.assertEqual(errors, expected)
 
     def test_b017(self):
         filename = Path(__file__).absolute().parent / "b017.py"
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
-        expected = self.errors(B017(24, 8), B017(26, 8), B017(28, 8))
+        expected = self.errors(
+            B017(26, 8),
+            B017(28, 8),
+            B017(30, 8),
+            B017(32, 8),
+            B017(73, 8),
+            B017(75, 8),
+            B017(77, 8),
+        )
         self.assertEqual(errors, expected)
 
     def test_b018_functions(self):
@@ -262,9 +278,23 @@ class BugbearTestCase(unittest.TestCase):
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
 
-        expected = [B018(line, 4) for line in range(16, 26)]
-        expected.append(B018(29, 4))
-        expected.append(B018(32, 4))
+        expected = [
+            B018(15, 4, vars=("Constant",)),
+            B018(16, 4, vars=("Constant",)),
+            B018(17, 4, vars=("Constant",)),
+            B018(18, 4, vars=("Constant",)),
+            B018(19, 4, vars=("Constant",)),
+            B018(20, 4, vars=("Constant",)),
+            B018(21, 4, vars=("Constant",)),
+            B018(22, 4, vars=("List",)),
+            B018(23, 4, vars=("Set",)),
+            B018(24, 4, vars=("Dict",)),
+            B018(28, 4, vars=("Constant",)),
+            B018(31, 4, vars=("Constant",)),
+            B018(32, 4, vars=("Tuple",)),
+            B018(33, 4, vars=("Tuple",)),
+        ]
+
         self.assertEqual(errors, self.errors(*expected))
 
     def test_b018_classes(self):
@@ -272,9 +302,23 @@ class BugbearTestCase(unittest.TestCase):
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
 
-        expected = [B018(line, 4) for line in range(17, 27)]
-        expected.append(B018(30, 4))
-        expected.append(B018(33, 4))
+        expected = [
+            B018(16, 4, vars=("Constant",)),
+            B018(17, 4, vars=("Constant",)),
+            B018(18, 4, vars=("Constant",)),
+            B018(19, 4, vars=("Constant",)),
+            B018(20, 4, vars=("Constant",)),
+            B018(21, 4, vars=("Constant",)),
+            B018(22, 4, vars=("Constant",)),
+            B018(23, 4, vars=("List",)),
+            B018(24, 4, vars=("Set",)),
+            B018(25, 4, vars=("Dict",)),
+            B018(29, 4, vars=("Constant",)),
+            B018(32, 4, vars=("Constant",)),
+            B018(33, 4, vars=("Tuple",)),
+            B018(34, 4, vars=("Tuple",)),
+        ]
+
         self.assertEqual(errors, self.errors(*expected))
 
     def test_b018_modules(self):
@@ -282,27 +326,58 @@ class BugbearTestCase(unittest.TestCase):
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
 
-        expected = [B018(line, 0) for line in range(9, 19)]
+        expected = [
+            B018(9, 0, vars=("Constant",)),
+            B018(10, 0, vars=("Constant",)),
+            B018(11, 0, vars=("Constant",)),
+            B018(12, 0, vars=("Constant",)),
+            B018(13, 0, vars=("Constant",)),
+            B018(14, 0, vars=("Constant",)),
+            B018(15, 0, vars=("Constant",)),
+            B018(16, 0, vars=("List",)),
+            B018(17, 0, vars=("Set",)),
+            B018(18, 0, vars=("Dict",)),
+            B018(19, 0, vars=("Tuple",)),
+            B018(20, 0, vars=("Tuple",)),
+        ]
+        self.assertEqual(errors, self.errors(*expected))
+
+    def test_b018_nested(self):
+        filename = Path(__file__).absolute().parent / "b018_nested.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+
+        expected = [
+            B018(3, 0, vars=("Constant",)),
+            B018(8, 4, vars=("Constant",)),
+            B018(14, 4, vars=("List",)),
+            B018(18, 8, vars=("Tuple",)),
+            B018(22, 12, vars=("Constant",)),
+            B018(25, 16, vars=("Constant",)),
+            B018(28, 20, vars=("Set",)),
+            B018(31, 20, vars=("Set",)),
+            B018(34, 20, vars=("Set",)),
+            B018(36, 24, vars=("Constant",)),
+            B018(40, 24, vars=("Constant",)),
+            B018(42, 24, vars=("Constant",)),
+        ]
         self.assertEqual(errors, self.errors(*expected))
 
     def test_b019(self):
         filename = Path(__file__).absolute().parent / "b019.py"
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
-
-        # AST Decorator column location for callable decorators changes in 3.7
-        col = 5 if sys.version_info >= (3, 7) else 4
         self.assertEqual(
             errors,
             self.errors(
+                B019(61, 5),
+                B019(64, 5),
+                B019(67, 5),
+                B019(70, 5),
                 B019(73, 5),
-                B019(77, 5),
-                B019(81, col),
-                B019(85, col),
-                B019(89, 5),
-                B019(93, 5),
-                B019(97, col),
-                B019(101, col),
+                B019(76, 5),
+                B019(79, 5),
+                B019(82, 5),
             ),
         )
 
@@ -347,29 +422,29 @@ class BugbearTestCase(unittest.TestCase):
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
         expected = self.errors(
-            B023(12, 29, vars=("x",)),
-            B023(13, 29, vars=("y",)),
-            B023(16, 15, vars=("x",)),
-            B023(28, 18, vars=("x",)),
+            B023(13, 29, vars=("x",)),
+            B023(14, 29, vars=("y",)),
+            B023(17, 15, vars=("x",)),
             B023(29, 18, vars=("x",)),
             B023(30, 18, vars=("x",)),
-            B023(31, 21, vars=("x",)),
-            B023(40, 33, vars=("x",)),
-            B023(42, 13, vars=("x",)),
-            B023(50, 29, vars=("a",)),
-            B023(51, 29, vars=("a_",)),
-            B023(52, 29, vars=("b",)),
-            B023(53, 29, vars=("c",)),
-            B023(61, 16, vars=("j",)),
-            B023(61, 20, vars=("k",)),
-            B023(68, 9, vars=("l",)),
-            B023(113, 23, vars=("x",)),
-            B023(114, 26, vars=("x",)),
-            B023(115, 36, vars=("x",)),
-            B023(116, 37, vars=("x",)),
-            B023(117, 36, vars=("x",)),
-            B023(168, 28, vars=("name",)),  # known false alarm
-            B023(171, 28, vars=("i",)),
+            B023(31, 18, vars=("x",)),
+            B023(32, 21, vars=("x",)),
+            B023(41, 33, vars=("x",)),
+            B023(43, 13, vars=("x",)),
+            B023(51, 29, vars=("a",)),
+            B023(52, 29, vars=("a_",)),
+            B023(53, 29, vars=("b",)),
+            B023(54, 29, vars=("c",)),
+            B023(62, 16, vars=("j",)),
+            B023(62, 20, vars=("k",)),
+            B023(69, 9, vars=("l",)),
+            B023(114, 23, vars=("x",)),
+            B023(115, 26, vars=("x",)),
+            B023(116, 36, vars=("x",)),
+            B023(117, 37, vars=("x",)),
+            B023(118, 36, vars=("x",)),
+            B023(169, 28, vars=("name",)),  # known false alarm
+            B023(172, 28, vars=("i",)),
         )
         self.assertEqual(errors, expected)
 
@@ -407,13 +482,14 @@ class BugbearTestCase(unittest.TestCase):
         self.assertEqual(
             errors,
             self.errors(
+                B026(15, 15),
                 B026(16, 15),
-                B026(17, 15),
-                B026(18, 26),
-                B026(19, 37),
-                B026(20, 15),
+                B026(17, 26),
+                B026(18, 37),
+                B026(19, 15),
+                B026(19, 25),
                 B026(20, 25),
-                B026(21, 25),
+                B026(21, 19),
             ),
         )
 
@@ -426,59 +502,212 @@ class BugbearTestCase(unittest.TestCase):
             B027(16, 4, vars=("empty_2",)),
             B027(19, 4, vars=("empty_3",)),
             B027(23, 4, vars=("empty_4",)),
-            B027(31 if sys.version_info >= (3, 8) else 30, 4, vars=("empty_5",)),
+            B027(31, 4, vars=("empty_5",)),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b028(self):
+        filename = Path(__file__).absolute().parent / "b028.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(B028(8, 0), B028(9, 0))
+        self.assertEqual(errors, expected)
+
+    def test_b029(self):
+        filename = Path(__file__).absolute().parent / "b029.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B029(8, 0),
+            B029(13, 0),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b030(self):
+        filename = Path(__file__).absolute().parent / "b030.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B030(3, 0),
+            B030(13, 0),
+            B030(18, 0),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b031(self):
+        filename = Path(__file__).absolute().parent / "b031.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B031(31, 36, vars=("section_items",)),
+            B031(35, 30, vars=("section_items",)),
+            B031(44, 36, vars=("section_items",)),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b032(self):
+        filename = Path(__file__).absolute().parent / "b032.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B032(9, 0),
+            B032(10, 0),
+            B032(12, 0),
+            B032(13, 0),
+            B032(16, 0),
+            B032(17, 0),
+            B032(18, 0),
+            B032(19, 0),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b033(self):
+        filename = Path(__file__).absolute().parent / "b033.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B033(6, 17, vars=("3",)),
+            B033(7, 23, vars=("'c'",)),
+            B033(8, 21, vars=("True",)),
+            B033(9, 20, vars=("None",)),
+            B033(10, 11, vars=("3.0",)),
+            B033(11, 11, vars=("True",)),
+            B033(12, 11, vars=("False",)),
+            B033(16, 4, vars=("True",)),
+            B033(18, 4, vars=("False",)),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b034(self):
+        filename = Path(__file__).absolute().parent / "b034.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B034(5, 24, vars=("sub", "count")),
+            B034(6, 24, vars=("sub", "count")),
+            B034(7, 24, vars=("sub", "count")),
+            B034(8, 25, vars=("subn", "count")),
+            B034(9, 25, vars=("subn", "count")),
+            B034(10, 25, vars=("subn", "count")),
+            B034(11, 25, vars=("split", "maxsplit")),
+            B034(12, 25, vars=("split", "maxsplit")),
+            B034(13, 25, vars=("split", "maxsplit")),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b035(self):
+        filename = Path(__file__).absolute().parent / "b035.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B035(6, 21, vars=("a",)),
+            B035(7, 21, vars=(1,)),
+            B035(19, 12, vars=("a",)),
+            B035(25, 21, vars=("CONST_KEY_VAR",)),
+            B035(35, 33, vars=("v3",)),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b036(self) -> None:
+        filename = Path(__file__).absolute().parent / "b036.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B036(4, 0),
+            B036(11, 0),
+            B036(20, 0),
+            B036(33, 0),
+            B036(50, 0),
+            B036(58, 0),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b037(self) -> None:
+        filename = Path(__file__).absolute().parent / "b037.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B037(4, 8),
+            B037(11, 12),
+            B037(15, 12),
+            B037(23, 8),
+            B037(29, 8),
+            B037(33, 8),
+        )
+        self.assertEqual(errors, expected)
+
+    def test_b908(self):
+        filename = Path(__file__).absolute().parent / "b908.py"
+        bbc = BugBearChecker(filename=str(filename))
+        errors = list(bbc.run())
+        expected = self.errors(
+            B908(7, 0),
+            B908(15, 8),
+            B908(21, 8),
+            B908(27, 8),
+            B017(37, 0),
+            B908(37, 0),
+            B908(41, 0),
+            B908(45, 0),
+            B017(56, 0),
         )
         self.assertEqual(errors, expected)
 
-    @unittest.skipIf(sys.version_info < (3, 8), "not implemented for <3.8")
     def test_b907(self):
         filename = Path(__file__).absolute().parent / "b907.py"
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
+        py39 = sys.version_info >= (3, 9)
+        py312 = sys.version_info >= (3, 12)
+
+        def on_py312(number):
+            """F-string nodes have column numbers set to 0 on <py312"""
+            return number if py312 else 0
+
         expected = self.errors(
-            B907(8, 0, vars=("var",)),
-            B907(9, 0, vars=("var",)),
-            B907(10, 0, vars=("var",)),
-            B907(12, 0, vars=("var",)),
-            B907(13, 0, vars=("var",)),
-            B907(14, 0, vars=("var",)),
-            B907(16, 0, vars=("'hello'",)),
-            B907(17, 0, vars=("foo()",)),
-            B907(20, 5, vars=("var",)),
-            B907(25, 5, vars=("var",)),
-            B907(31, 0, vars=("var",)),
-            B907(32, 0, vars=("var",)),
-            B907(33, 0, vars=("var",)),
-            B907(33, 0, vars=("var2",)),
-            B907(34, 0, vars=("var",)),
-            B907(34, 0, vars=("var2",)),
-            B907(35, 0, vars=("var",)),
-            B907(35, 0, vars=("var2",)),
-            B907(38, 0, vars=("var2",)),
-            B907(41, 0, vars=("var",)),
-            B907(42, 0, vars=("var.__str__",)),
-            B907(43, 0, vars=("var.__str__.__repr__",)),
-            B907(44, 0, vars=("3 + 5" if sys.version_info >= (3, 9) else "BinOp",)),
-            B907(45, 0, vars=("foo()",)),
-            B907(46, 0, vars=("None",)),
-            B907(47, 0, vars=("..." if sys.version_info >= (3, 9) else "Ellipsis",)),
-            B907(48, 0, vars=("True",)),
-            B907(51, 0, vars=("var",)),
-            B907(52, 0, vars=("var",)),
-            B907(53, 0, vars=("var",)),
-            B907(54, 0, vars=("var",)),
-            B907(57, 0, vars=("var",)),
-            B907(60, 0, vars=("var",)),
-            B907(64, 0, vars=("var",)),
-            B907(66, 0, vars=("var",)),
-            B907(68, 0, vars=("var",)),
+            B907(8, on_py312(9), vars=("var",)),
+            B907(9, on_py312(3), vars=("var",)),
+            B907(10, on_py312(9), vars=("var",)),
+            B907(12, on_py312(9), vars=("var",)),
+            B907(13, on_py312(3), vars=("var",)),
+            B907(14, on_py312(9), vars=("var",)),
+            B907(16, on_py312(5), vars=("'hello'",)),
+            B907(17, on_py312(5), vars=("foo()",)),
+            # Multiline f-strings have lineno changes as well as colno changes on py312+
+            B907(21 if py312 else 20, 7 if py312 else 5, vars=("var",)),
+            B907(26 if py312 else 25, 7 if py312 else 5, vars=("var",)),
+            B907(31, on_py312(12), vars=("var",)),
+            B907(32, on_py312(3), vars=("var",)),
+            B907(33, on_py312(3), vars=("var",)),
+            B907(33, on_py312(29), vars=("var2",)),
+            B907(34, on_py312(3), vars=("var",)),
+            B907(34, on_py312(15), vars=("var2",)),
+            B907(35, on_py312(3), vars=("var",)),
+            B907(35, on_py312(10), vars=("var2",)),
+            B907(38, on_py312(13), vars=("var2",)),
+            B907(41, on_py312(3), vars=("var",)),
+            B907(42, on_py312(3), vars=("var.__str__",)),
+            B907(43, on_py312(3), vars=("var.__str__.__repr__",)),
+            B907(44, on_py312(3), vars=("3 + 5" if py39 else "BinOp",)),
+            B907(45, on_py312(3), vars=("foo()",)),
+            B907(46, on_py312(3), vars=("None",)),
+            B907(47, on_py312(3), vars=("..." if py39 else "Ellipsis",)),
+            B907(48, on_py312(3), vars=("True",)),
+            B907(51, on_py312(3), vars=("var",)),
+            B907(52, on_py312(3), vars=("var",)),
+            B907(53, on_py312(3), vars=("var",)),
+            B907(54, on_py312(3), vars=("var",)),
+            B907(57, on_py312(3), vars=("var",)),
+            B907(60, on_py312(3), vars=("var",)),
+            B907(64, on_py312(5), vars=("var",)),
+            B907(66, on_py312(3), vars=("var",)),
+            B907(68, on_py312(3), vars=("var",)),
         )
         self.assertEqual(errors, expected)
 
     # manual permutations to save overhead when doing >60k permutations
     # see format spec at
     # https://docs.python.org/3/library/string.html#format-specification-mini-language
-    @unittest.skipIf(sys.version_info < (3, 8), "not implemented for <3.8")
     def test_b907_format_specifier_permutations(self):
         visitor = BugBearVisitor(filename="", lines="")
 
@@ -534,7 +763,7 @@ class BugbearTestCase(unittest.TestCase):
         filename = Path(__file__).absolute().parent / "b901.py"
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
-        self.assertEqual(errors, self.errors(B901(9, 8), B901(36, 4)))
+        self.assertEqual(errors, self.errors(B901(8, 8), B901(35, 4)))
 
     def test_b902(self):
         filename = Path(__file__).absolute().parent / "b902.py"
@@ -543,20 +772,52 @@ class BugbearTestCase(unittest.TestCase):
         self.assertEqual(
             errors,
             self.errors(
-                B902(29, 17, vars=("'i_am_special'", "instance", "self")),
-                B902(32, 30, vars=("'cls'", "instance", "self")),
-                B902(35, 4, vars=("(none)", "instance", "self")),
-                B902(39, 12, vars=("'self'", "class", "cls")),
-                B902(42, 22, vars=("*args", "instance", "self")),
-                B902(48, 30, vars=("**kwargs", "instance", "self")),
-                B902(51, 32, vars=("*, self", "instance", "self")),
-                B902(54, 44, vars=("*, self", "instance", "self")),
-                B902(68, 17, vars=("'self'", "metaclass instance", "cls")),
-                B902(72, 20, vars=("'cls'", "metaclass class", "metacls")),
+                B902(22, 17, vars=("'i_am_special'", "instance", "self")),
+                B902(24, 30, vars=("'cls'", "instance", "self")),
+                B902(26, 4, vars=("(none)", "instance", "self")),
+                B902(29, 12, vars=("'self'", "class", "cls")),
+                B902(31, 22, vars=("*args", "instance", "self")),
+                B902(36, 30, vars=("**kwargs", "instance", "self")),
+                B902(38, 32, vars=("*, self", "instance", "self")),
+                B902(40, 44, vars=("*, self", "instance", "self")),
+                B902(52, 17, vars=("'self'", "metaclass instance", "cls")),
+                B902(55, 20, vars=("'cls'", "metaclass class", "metacls")),
+            ),
+        )
+
+    def test_b902_extended(self):
+        filename = Path(__file__).absolute().parent / "b902_extended.py"
+
+        mock_options = Namespace(
+            classmethod_decorators=["mylibrary.makeclassmethod", "validator"],
+            select=["B902"],
+        )
+        bbc = BugBearChecker(filename=str(filename), options=mock_options)
+        errors = list(bbc.run())
+
+        self.assertEqual(
+            errors,
+            self.errors(
+                B902(5, 22, vars=("'self'", "class", "cls")),
+                B902(8, 28, vars=("'self'", "class", "cls")),
+                B902(11, 30, vars=("'self'", "class", "cls")),
+                B902(14, 27, vars=("'cls'", "instance", "self")),
+                B902(18, 13, vars=("'cls'", "instance", "self")),
+                B902(22, 13, vars=("'cls'", "instance", "self")),
+                B902(26, 13, vars=("'cls'", "instance", "self")),
+                B902(30, 13, vars=("'cls'", "instance", "self")),
+                # metaclass
+                B902(59, 22, vars=("'cls'", "metaclass class", "metacls")),
+                B902(62, 28, vars=("'cls'", "metaclass class", "metacls")),
+                B902(65, 30, vars=("'cls'", "metaclass class", "metacls")),
+                B902(68, 27, vars=("'metacls'", "metaclass instance", "cls")),
+                B902(72, 13, vars=("'metacls'", "metaclass instance", "cls")),
+                B902(76, 13, vars=("'metacls'", "metaclass instance", "cls")),
+                B902(80, 13, vars=("'metacls'", "metaclass instance", "cls")),
+                B902(84, 13, vars=("'metacls'", "metaclass instance", "cls")),
             ),
         )
 
-    @unittest.skipIf(sys.version_info < (3, 8), "requires 3.8+")
     def test_b902_py38(self):
         filename = Path(__file__).absolute().parent / "b902_py38.py"
         bbc = BugBearChecker(filename=str(filename))
@@ -564,16 +825,16 @@ class BugbearTestCase(unittest.TestCase):
         self.assertEqual(
             errors,
             self.errors(
-                B902(29, 17, vars=("'i_am_special'", "instance", "self")),
-                B902(32, 30, vars=("'cls'", "instance", "self")),
-                B902(35, 4, vars=("(none)", "instance", "self")),
-                B902(39, 12, vars=("'self'", "class", "cls")),
-                B902(42, 22, vars=("*args", "instance", "self")),
-                B902(48, 30, vars=("**kwargs", "instance", "self")),
-                B902(51, 32, vars=("*, self", "instance", "self")),
-                B902(54, 44, vars=("*, self", "instance", "self")),
-                B902(68, 17, vars=("'self'", "metaclass instance", "cls")),
-                B902(72, 20, vars=("'cls'", "metaclass class", "metacls")),
+                B902(22, 17, vars=("'i_am_special'", "instance", "self")),
+                B902(24, 30, vars=("'cls'", "instance", "self")),
+                B902(26, 4, vars=("(none)", "instance", "self")),
+                B902(29, 12, vars=("'self'", "class", "cls")),
+                B902(31, 22, vars=("*args", "instance", "self")),
+                B902(36, 30, vars=("**kwargs", "instance", "self")),
+                B902(38, 32, vars=("*, self", "instance", "self")),
+                B902(40, 44, vars=("*, self", "instance", "self")),
+                B902(52, 17, vars=("'self'", "metaclass instance", "cls")),
+                B902(55, 20, vars=("'cls'", "metaclass class", "metacls")),
             ),
         )
 
@@ -581,7 +842,7 @@ class BugbearTestCase(unittest.TestCase):
         filename = Path(__file__).absolute().parent / "b903.py"
         bbc = BugBearChecker(filename=str(filename))
         errors = list(bbc.run())
-        self.assertEqual(errors, self.errors(B903(32, 0), B903(38, 0)))
+        self.assertEqual(errors, self.errors(B903(31, 0), B903(37, 0)))
 
     def test_b904(self):
         filename = Path(__file__).absolute().parent / "b904.py"
@@ -608,6 +869,8 @@ class BugbearTestCase(unittest.TestCase):
             B905(4, 15),
             B905(5, 4),
             B905(6, 0),
+            B905(21, 0),
+            B905(22, 0),
         ]
         self.assertEqual(errors, self.errors(*expected))
 
@@ -631,6 +894,8 @@ class BugbearTestCase(unittest.TestCase):
                 B950(12, 103, vars=(103, 79)),
                 B950(14, 103, vars=(103, 79)),
                 B950(21, 97, vars=(97, 79)),
+                B950(35, 104, vars=(104, 79)),
+                B950(36, 104, vars=(104, 79)),
             ),
         )
 
@@ -647,6 +912,8 @@ class BugbearTestCase(unittest.TestCase):
                 B950(12, 103, vars=(103, 79)),
                 B950(14, 103, vars=(103, 79)),
                 B950(21, 97, vars=(97, 79)),
+                B950(35, 104, vars=(104, 79)),
+                B950(36, 104, vars=(104, 79)),
             ),
         )
 
@@ -665,6 +932,8 @@ class BugbearTestCase(unittest.TestCase):
                 B950(12, 103, vars=(103, 79)),
                 B950(14, 103, vars=(103, 79)),
                 B950(21, 97, vars=(97, 79)),
+                B950(35, 104, vars=(104, 79)),
+                B950(36, 104, vars=(104, 79)),
             ),
         )
 
@@ -682,8 +951,7 @@ class BugbearTestCase(unittest.TestCase):
         filename = Path(__file__).absolute().parent.parent / "bugbear.py"
         proc = subprocess.run(
             ["flake8", str(filename)],
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
+            capture_output=True,
             timeout=60,
         )
         self.assertEqual(proc.returncode, 0, proc.stdout.decode("utf8"))
@@ -694,16 +962,42 @@ class BugbearTestCase(unittest.TestCase):
         filename = Path(__file__).absolute()
         proc = subprocess.run(
             ["flake8", str(filename)],
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
+            capture_output=True,
             timeout=60,
         )
         self.assertEqual(proc.returncode, 0, proc.stdout.decode("utf8"))
         self.assertEqual(proc.stdout, b"")
         self.assertEqual(proc.stderr, b"")
 
+    def test_b909(self):
+        filename = Path(__file__).absolute().parent / "b909.py"
+        mock_options = Namespace(select=[], extend_select=["B909"])
+        bbc = BugBearChecker(filename=str(filename), options=mock_options)
+        errors = list(bbc.run())
+        print(errors)
+        expected = [
+            B909(11, 8),
+            B909(26, 8),
+            B909(27, 8),
+            B909(41, 8),
+            B909(47, 8),
+            B909(56, 8),
+            B909(57, 8),
+            B909(58, 8),
+            B909(59, 8),
+            B909(60, 8),
+            B909(61, 8),
+            B909(62, 8),
+            B909(63, 8),
+            B909(74, 8),
+        ]
+        self.assertEqual(errors, self.errors(*expected))
+
 
 class TestFuzz(unittest.TestCase):
+    from hypothesis import HealthCheck, given, settings
+    from hypothesmith import from_grammar
+
     @settings(suppress_health_check=[HealthCheck.too_slow])
     @given(from_grammar().map(ast.parse))
     def test_does_not_crash_on_any_valid_code(self, syntax_tree):
